"use strict";

var _ast = require("@webassemblyjs/ast");

var _wasmParser = require("@webassemblyjs/wasm-parser");

var _path = _interopRequireDefault(require("path"));

var _fs = require("fs");

var _estraverse = require("estraverse");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var decoderOpts = {
  ignoreCodeSection: true,
  ignoreDataSection: true
};

function getExportsFromWasm(file) {
  var exports = [];
  var binary = (0, _fs.readFileSync)(file, null);
  var ast = (0, _wasmParser.decode)(binary, decoderOpts);
  (0, _ast.traverse)(ast, {
    ModuleExport: function ModuleExport(_ref) {
      var node = _ref.node;
      exports.push(node.name);
    }
  });
  return exports;
}

function getUsageIn(body, on) {
  var used = [];
  (0, _estraverse.traverse)(body, {
    enter: function enter(node) {
      if (node.type === "MemberExpression") {
        var object = node.object,
            property = node.property;

        if (object.name === on) {
          used.push(property.name);
        }
      }
    }
  });
  return used;
}

module.exports = {
  create: function create(context) {
    return {
      /**
       * `import(x).then(f)`
       */
      MemberExpression: function MemberExpression(node) {
        var object = node.object,
            property = node.property;

        if (object.type !== "CallExpression") {
          return;
        }

        if (object.callee.type !== "Import") {
          return;
        }

        if (property.name !== "then") {
          return;
        }

        var dirname = _path["default"].dirname(context.getFilename());

        var source = object.arguments[0].value;

        var file = _path["default"].join(dirname, source);

        if ((0, _fs.existsSync)(file) === false) {
          return context.report({
            node: object,
            message: JSON.stringify(file) + " is not a valid WASM file"
          });
        }

        var exports = getExportsFromWasm(file); // get binding in `then(x)`

        var _node$parent$argument = _slicedToArray(node.parent.arguments, 1),
            fn = _node$parent$argument[0];

        var _fn$params = _slicedToArray(fn.params, 1),
            binding = _fn$params[0];

        var used = [];

        if (binding.type === "Identifier") {
          used = getUsageIn(fn.body, binding.name);
        }

        if (binding.type === "ObjectPattern") {
          used = binding.properties.map(function (prop) {
            return prop.value.name;
          });
        }

        used.forEach(function (name) {
          var exportExists = exports.indexOf(name) !== -1;

          if (exportExists === false) {
            context.report({
              node: node,
              message: JSON.stringify(name) + " is not exported"
            });
          }
        });
      }
    };
  }
};